"""Specialized forms provided by Djblets.

This module provides a centralized place to load Djblets's utility forms.
From here, you can import:

* :py:class:`~djblets.forms.forms.key_value_form.KeyValueForm`
"""

from __future__ import unicode_literals

from djblets.forms.forms.key_value_form import KeyValueForm


__all__ = [
    'KeyValueForm',
]
